using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.IO;
using System.Collections;
using System.Collections.Specialized;

namespace GroupFinity.Greplace.MediaInfoPlugin
{
	public enum MediaPropertyPage
	{
		ID3v1,
		ID3v2,
		RiffSIF,
	}

	/*
	public class MediaInfoName
	{
		// Common
		public static readonly string
			FileName = "FILE",
			Extension = "FEXT",
			Path = "PATH",
			ByteSize = "SIZ1",
			KBSize = "SIZK",
			MBSize = "SIZM";

		// MP3
		public static readonly string
			AudioFormat = "AFMT",
			Duration = "TIME",
			TitleV1 = "INAM_v1",
			TitleV2 = "INAM_v2",
			TitleRiff = "INAM_rmp",
			ArtistV1 = "IART_v1",
			ArtistV2 = "IART_v2",
			ArtistRiff = "IART_rmp",
			AlbumV1 = "IPRD_v1",
			AlbumV2 = "IPRD_v2",
			AlbumRiff = "IPRD_rmp",
			CommentV1 = "ICMT_v1",
			CommentV2 = "ICMT_v2",
			CommentRiff = "ICMT_rmp",
			CreationTimeV1 = "ICRD_v1",
			CreationTimeV2 = "ICRD_v2",
			CreationTimeRiff = "ICRD_rmp",
			GenreV1 = "IGNR_v1",
			GenreV2 = "IGNR_v2",
			GenreRiff = "IGNR_rmp",
			CopyrightV2 = "ICOP_v2",
			CopyrightRiff = "ICOP_rmp",
			SoftwareV2 = "ISFT_v2",
			SoftwareRiff = "ISFT_rmp",
			ComposerV2 = "COMP_v2",
			OriginalArtistV2 = "OART_v2",
			UrlV2 = "URL_v2",
			SourceRiff = "ISRC_rmp",
			EnginieerRiff = "IENG_rmp",
			TrackV1 = "TRACK_v1",
			TrackV2 = "TRACK_v2";

		// Wave
		public static readonly string
			Title = "INAM",
			TitleWave = "ISBJ",
			Artist = "IART",
			Album = "IPRD",
			Comment = "ICMT",
			CreationTime = "ICRD",
			Jenre = "IGNR",
			Copyright = "ICOP",
			Software = "ISFT",
			Source = "ISRC",
			Enginier = "IENG";

		// AVI
		public static readonly string 
		VideoFormat = "VFMT",
		AviVersion = "AVIV";

		// VQF

		// WM
		public static readonly string 
			AlubmUrl = "URL1",
			RelatedUrl = "URL2";

		// OGG

		// APE
	}
	*/

	public enum MediaType
	{
		Unknown,
		Mp3,
		Wav,
		Avi,
		Vqr,
		WindowsMedia,
		OGG,
		APE
	}

	[Flags()]
	public enum TagTypes
	{
		ID3v1 = 1,
		ID3v2 = 2,
		RiffSIF = 4,
	}

	public class MediaInfo
	{
		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_GetVer();
		public static int Version{ get{ return mp3infp_GetVer(); } }

		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_ViewProp( IntPtr hWnd, string szFileName, int dwPage );
		public static bool ShowPropertyPage( string fileName, MediaPropertyPage page, Form parentForm ){ return mp3infp_ViewProp( parentForm==null ? IntPtr.Zero : parentForm.Handle, fileName, (int)page )!=0; }

		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_Load( IntPtr hWnd, string szFileName );
		public static bool Load( string fileName, Form parentForm ){ return mp3infp_Load( parentForm==null ? IntPtr.Zero : parentForm.Handle, fileName )!=-1; }

		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_GetType();
		public static MediaType GetMediaType(){return (MediaType)mp3infp_GetType();}

		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_GetValue( string name, out IntPtr buffer );
		public static string GetMediaInfo(string name)
		{
			IntPtr result; 
			mp3infp_GetValue( name, out result );
			return Marshal.PtrToStringAnsi( result );
		}

		[DllImport("mp3infp", CharSet = CharSet.Ansi)]
		extern private static int mp3infp_mp3_GetTagType();
		public static TagTypes GetTagTypes(){ return (TagTypes)mp3infp_mp3_GetTagType(); }

	}

	[LocalizedEvaluator("mediaEvaluator","media")]
	[LocalizedEvaluatorUsage("fileName", "${media:file-name}" )]
	[LocalizedEvaluatorUsage("fileExt", "${media:file-ext}" )]
	[LocalizedEvaluatorUsage("filePath", "${media:file-path}" )]
	[LocalizedEvaluatorUsage("fileSize", "${media:file-size}" )]
	[LocalizedEvaluatorUsage("type", "${media:type}" )]
	[LocalizedEvaluatorUsage("format", "${media:format}" )]
	[LocalizedEvaluatorUsage("duration", "${media:duration}" )]
	[LocalizedEvaluatorUsage("title", "${media:title}" )]
	[LocalizedEvaluatorUsage("artist", "${media:artist}" )]
	[LocalizedEvaluatorUsage("album", "${media:album}" )]
	[LocalizedEvaluatorUsage("comment", "${media:comment}" )]
	[LocalizedEvaluatorUsage("creationTime", "${media:creation-time}" )]
	[LocalizedEvaluatorUsage("genre", "${media:genre}" )]
	[LocalizedEvaluatorUsage("copyright", "${media:copyright}" )]
	[LocalizedEvaluatorUsage("software", "${media:software}" )]
	[LocalizedEvaluatorUsage("composer", "${media:composer}" )]
	[LocalizedEvaluatorUsage("originalArtist", "${media:artist-original}" )]
	[LocalizedEvaluatorUsage("url", "${media:url}" )]
	[LocalizedEvaluatorUsage("source", "${media:source}" )]
	[LocalizedEvaluatorUsage("enginieer", "${media:enginieer}" )]
	[LocalizedEvaluatorUsage("track", "${media:track}" )]
	[LocalizedEvaluatorUsage("subject", "${media:subject}" )]
	[LocalizedEvaluatorUsage("aviVersion", "${media:avi-version}" )]
	[LocalizedEvaluatorUsage("albumUrl", "${media:url-album}" )]
	[LocalizedEvaluatorUsage("relatedUrl", "${media:url-related}" )]
	public class MediaInformationEvaluator : IEvaluator
	{
		static Hashtable stringTable;
		static MediaInformationEvaluator()
		{
			stringTable = new Hashtable();

			stringTable["file-name"] = "FILE";
			stringTable["file-ext"] = "FEXT";
			stringTable["file-path"] = "PATH";
			stringTable["file-size"] = "SIZ1";
			stringTable["file-size-byte"] = "SIZ1";
			stringTable["file-size-kb"] = "SIZK";
			stringTable["file-size-mb"] = "SIZM";

			stringTable["format"] = "AFMT";
			stringTable["format-video"] = "VFMT";
			stringTable["format-audio"] = "AFMT";
			stringTable["duration"] = "TIME";
			stringTable["title"] = "INAM";
			stringTable["artist"] = "IART";
			stringTable["album"] = "IPRD";
			stringTable["comment"] = "ICMT";
			stringTable["creation-time"] = "ICRD";
			stringTable["genre"] = "IGNR";
			stringTable["copyright"] = "ICOP";
			stringTable["software"] = "ISFT";
			stringTable["composer"] = "COMP";
			stringTable["artist-original"] = "OART";
			stringTable["url"] = "URL";
			stringTable["source"] = "ISRC";
			stringTable["enginieer"] = "IENG";
			stringTable["track"] = "TRACK";
			stringTable["subject"] = "ISBJ";
			stringTable["avi-version"] = "AVIV";

			stringTable["url-album"] = "URL1";
			stringTable["url-related"] = "URL2";
		}
		public string Evaluate( IGreppedText text, Match match, string input)
		{	
			try
			{
				int separator = input.IndexOf( '|' );
				string defaultValue = separator<0 ? string.Empty : input.Substring( separator+1 );
				if ( separator>=0 ) input = input.Substring( 0, separator );

				int colon = input.IndexOf( ':' );
				string path;
				if ( colon<0 )
				{
					if ( !(text.Parent is IGreppedFile) ) return string.Empty;
					path = GetFullPathOf( text.Parent );
				}
				else
				{
					path = input.Substring( colon+1 );
					input = input.Substring( 0, colon );
				}

				if ( !MediaInfo.Load( path, null ) ) return defaultValue;

				if ( input=="type" )
				{
					switch( MediaInfo.GetMediaType() )
					{
						case MediaType.Unknown: return defaultValue;
						case MediaType.Mp3: return "MP3";
						case MediaType.Wav: return "WAVE";
						case MediaType.Avi: return "AVI";
						case MediaType.Vqr: return "VQR";
						case MediaType.WindowsMedia: return "Windows Media";
						case MediaType.OGG: return "OGG";
						case MediaType.APE: return "APE";
						default: return defaultValue;
					}
				}

				string result;

				if ( input=="format" )
				{
					string name = "AFMT";
					result = MediaInfo.GetMediaInfo( name );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_rmp" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v2" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v1" );
					string temp = result;
					name = "VFMT";
					result = MediaInfo.GetMediaInfo( name );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_rmp" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v2" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v1" );

					if ( temp!=null && temp==string.Empty && result!=null && result==string.Empty ) temp += " ";
					result = temp + result;
				}
				else
				{
					string name = input;
					string o = stringTable[input] as string;
					if ( o!=null ) name = o;

					result = MediaInfo.GetMediaInfo( name );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_rmp" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v2" );
					if ( result==null || result==string.Empty ) result = MediaInfo.GetMediaInfo( name+"_v1" );
					if ( result!=null && result!=string.Empty )
					{
						if ( input=="file-size" ) result = result.Replace( ",", "" );
						else if ( input.StartsWith("duration") )
						{
							separator = result.IndexOf( "(" );
							if ( separator>=0 ) result = result.Substring( separator+1 );
							separator = result.IndexOf( "sec)" );
							if ( separator>=0 ) result = result.Substring( 0, separator );

							if ( input=="duration-sec" ) result = int.Parse( result ).ToString( "N" );
							else if ( input=="duration-format" ) result = TimeSpan.FromSeconds( double.Parse( result ) ).ToString();
						}
					}
				}

				if ( result==null || result==string.Empty ) result = defaultValue;

				return result;
			}
			catch{ return string.Empty; }
		}
		private static string GetFullPathOf( IGreppedPath path )
		{
			return path.Parent==null ? path.Name : Path.Combine( GetFullPathOf( path.Parent ), path.Name );
		}
	}
}
