using System;
using System.Text.RegularExpressions;

namespace GroupFinity.Greplace.StandardPlugin
{
	[LocalizedEvaluator("matchEvaluator", "match")]
	[LocalizedEvaluatorUsage("matchNumber", "${match:number}" )]
	[LocalizedEvaluatorUsage("matchLine", "${match:line}" )]
	[LocalizedEvaluatorUsage("matchColumn", "${match:column}" )]
	[LocalizedEvaluatorUsage("matchIndex", "${match:index}" )]
	[LocalizedEvaluatorUsage("parentNumber", "${match:parent-number}" )]
	[LocalizedEvaluatorUsage("fileName", "${match:parent-name}" )]
	[LocalizedEvaluatorUsage("filePath", "${match:parent-path}" )]
	public class MatchEvaluator : IEvaluator
	{
		public string Evaluate( IGreppedText text, Match match, string input)
		{
			IGreppedData data = text;
			while( input.StartsWith( "parent-" ) )
			{
				data = data.Parent;
				input = input.Substring( "parent-".Length );
			}

			if ( data==text )
			{
				if ( input=="line" ) return text.Line.ToString();
				else if ( input=="column" ) return text.Column.ToString();
				else if ( input=="index" ) return text.Index.ToString();
				else if ( input=="number" ) return text.Number.ToString();
				else return string.Empty;
			}
			else
			{
				IGreppedPath path = data as IGreppedPath;

				if ( input=="number" ) return data.Number.ToString();
				else if ( input=="name" ) return path.Name;
				else if ( input=="path" ) return GetFullPathOf( path );
				else return string.Empty;
			}
		}
		
		private static string GetFullPathOf( IGreppedPath path )
		{
			return path.Parent==null ? path.Name : System.IO.Path.Combine( GetFullPathOf( path.Parent ), path.Name );
		}
	}
	
}
